@@ -386,12 +386,11 @@ NumericMatrix Wijs_mat52_sym_cpp(NumericMatrix Mu, NumericVector sigma){
         b2 = b*b;
         
         if(i == j){
-          Wijs(i,j) *= (exp(-2*sqrt(5.)*a/t)*(63*t2*t2*exp(2*sqrt(5.)*a/t)-50*a2*a2-16*5*sqrt(5.)*t*a2*a-270*t2*a2-18*5*sqrt(5.)*t2*t*a-63*t2*t2)-exp(-2*sqrt(5.)/t)*((t*(t*(10*(5*a2-27*a+27)+9*t*(7*t-5*sqrt(5.)*(2*a-2))+10*a*(22*a-27))-8*5*sqrt(5.)*(a-1)*(a-1)*(2*a-2))+50*(a-2)*(a-1)*(a-1)*a+50*(a-1)*(a-1))*exp(2*sqrt(5.)*a/t)-63*t2*t2*exp(2*sqrt(5.)/t)))/(36*sqrt(5.)*t2*t);;
-        }else{
-          p1 = (2*t2*(63*t2 + 9*5.*sqrt(5.)*b*t-9*5.*sqrt(5.)*a*t+50*b2-100*a*b+50*a2)*exp(2*sqrt(5.)*a/t)-63*t2*t2-9*5.*sqrt(5.)*(b+a)*t*t2-10*(5*b2+17*a*b+5*a2)*t2-8*5.*sqrt(5.)*a*b*(b+a)*t-50*a2*b2)*exp(-sqrt(5.)*(b+a)/t)/(36*sqrt(5.)*t*t2);
+          Wijs(i,j) *= (63*t2*t2+exp(-2*sqrt(5.)*a/t)*(-50*a2*a2-16*5*sqrt(5.)*t*a2*a-270*t2*a2-18*5*sqrt(5.)*t2*t*a-63*t2*t2) - ((t*(t*(10*(5*a2-27*a+27)+9*t*(7*t-5*sqrt(5.)*(2*a-2))+10*a*(22*a-27))-8*5*sqrt(5.)*(a-1)*(a-1)*(2*a-2))+50*(a-2)*(a-1)*(a-1)*a+50*(a-1)*(a-1)) * exp(-2*sqrt(5.)*(1-a)/t) - 63*t2*t2))/(36*sqrt(5.)*t2*t);
+         }else{
+          p1 = (2*t2*(63*t2 + 9*5.*sqrt(5.)*b*t-9*5.*sqrt(5.)*a*t+50*b2-100*a*b+50*a2)* exp(-sqrt(5.)*(b-a)/t)+(-63*t2*t2-9*5.*sqrt(5.)*(b+a)*t*t2-10*(5*b2+17*a*b+5*a2)*t2 -8*5.*sqrt(5.)*a*b*(b+a)*t-50*a2*b2)* exp(-sqrt(5.)*(b+a)/t)) /(36*sqrt(5.)*t*t2);
           p3 = (b-a)*(54*t2*t2+(54*sqrt(5.)*b-54*sqrt(5.)*a)*t*t2+(105*b2-210*a*b+105*a2)*t2+(3*5.*sqrt(5.)*b2*b-9*5.*sqrt(5.)*a*b2+9*5.*sqrt(5.)*a2*b-3*5.*sqrt(5.)*a2*a)*t+5*b2*b2-20*a*b2*b+30*a2*b2-20*a2*a*b+5*a2*a2)*exp(sqrt(5.)*(a-b)/t)/(54*t2*t2);
-          p4 = -((t*(t*(9*t*(7*t-5.*sqrt(5.)*(b+a-2))+10*b*(5*b+17*a-27)+10*(5*a2-27*a+27))-8*5.*sqrt(5.)*(a-1)*(b-1)*(b+a-2))+50*(a-1)*(a-1)*(b-2)*b+50*(a-1)*(a-1))*exp(2*sqrt(5.)*b/t))*exp(-sqrt(5.)*(b-a+2)/t)/(36*sqrt(5.)*t*t2);
-          
+          p4 = -((t*(t*(9*t*(7*t-5.*sqrt(5.)*(b+a-2))+10*b*(5*b+17*a-27)+10*(5*a2-27*a+27))-8*5.*sqrt(5.)*(a-1)*(b-1)*(b+a-2))+50*(a-1)*(a-1)*(b-2)*b+50*(a-1)*(a-1))*exp(-sqrt(5.)*(-b-a+2)/t))/(36*sqrt(5.)*t*t2);
           Wijs(j,i) = Wijs(i,j) *= p1 + p3 + p4;
         }
         
